#!/usr/bin/perl
#
# (C) 2001-2002
#	Domsodi Gergely	<doome@uhulinux.hu>
#	Pozsar Balazs	<pozsy@uhulinux.hu>
#
# This file is originally part of the UHU Linux distribution.
# Distributable under GPL v2.
#

# ----- Pragmatic modules -----------------------------------------------------
use strict;
use warnings;

# -----------------------------------------------------------------------------
package UHU::xml;

use XML::LibXML;

use UHU::common;
use UHU::utf8;

# ----- Exporter --------------------------------------------------------------
require Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(
load_xml save_xml
new_child get_children
get_text set_text
remove_node
);

# ----- General helper subs ---------------------------------------------------
sub load_xml($) {
	my ($filename) = @_;

	my $parser = XML::LibXML->new() or return;

	my $dom  = $parser->parse_file($filename);
	my $root = $dom->getDocumentElement;

	$root->getType == (XML_ELEMENT_NODE) ? $root : undef;
}

sub save_xml($$;$) {
	my ($root, $filename, $level) = @_;

	put $root->getOwnerDocument->toString($level or 1), $filename;
}

sub new_child($$) {
	my ($parent, $name) = @_;

	my $child = XML::LibXML::Element->new($name);
	$parent->appendChild($child);

	$child;
}

sub get_children($;$) {
	my ($elem, $type) = @_;

	return unless $elem;
	return unless $elem->getType == (XML_ELEMENT_NODE);

	$type = (XML_ELEMENT_NODE) unless $type;

	map {$_->getType == $type ? $_ : ()} $elem->getChildnodes;
}

sub get_text($;$) {
	my ($elem, $charset) = @_;

	f8(
		join ('',
			map {$_->nodeValue or ()} get_children($elem, (XML_TEXT_NODE))),
		$charset
	);
}

sub set_text($$;$) {
	my ($elem, $text, $charset) = @_;

	$elem->removeChild($_) foreach get_children($elem, (XML_TEXT_NODE));
	$elem->appendText(t8($text, $charset));
	$elem;
}

sub remove_node($) {
	my ($node) = @_;

	my $parent = $node->getParentNode;
	die "Nincs szuloje a torleshez: $node" unless $parent;

	$parent->removeChild($node);
}

# ----- Beatiful perl :) ------------------------------------------------------
1;
